Package: unmarked
Version: 1.0.0
Date: 2020-05-02
Type: Package
Title: Models for Data from Unmarked Animals
Authors@R: c(
    person("Richard", "Chandler", role="aut"), 
    person("Ken", "Kellner", role="aut"),
    person("Ian", "Fiske", role="aut"), 
    person("David", "Miller", role="aut"),
    person("Andy", "Royle", email="aroyle@usgs.gov", role=c("cre", "aut")),
    person("Jeff", "Hostetler", role="aut"),
    person("Rebecca", "Hutchinson", role="aut"),
    person("Adam", "Smith", role="aut"),
    person("Marc", "Kery", role="ctb"),
    person("Mike", "Meredith", role="ctb"),
    person("Auriel", "Fournier", role="ctb"),
    person("Ariel", "Muldoon", role="ctb"),
    person("Chris", "Baker", role="ctb")
    )
Depends: R (>= 2.12.0), methods, lattice
Imports: graphics, stats, utils, parallel, plyr, raster, Matrix, MASS,
        Rcpp (>= 0.8.0)
Description: Fits hierarchical models of animal abundance and occurrence to data collected using survey methods such as point counts, site occupancy sampling, distance sampling, removal sampling, and double observer sampling. Parameters governing the state and observation processes can be modeled as functions of covariates.
License: GPL (>= 3)
LazyLoad: yes
LazyData: yes
Collate: 'classes.R' 'unmarkedEstimate.R' 'mapInfo.R' 'unmarkedFrame.R'
        'unmarkedFit.R' 'utils.R' 'getDesign.R' 'colext.R' 'distsamp.R'
        'multinomPois.R' 'occu.R' 'occuRN.R' 'occuMulti.R' 'pcount.R'
        'gmultmix.R' 'pcountOpen.R' 'gdistsamp.R' 'unmarkedFitList.R'
        'unmarkedLinComb.R' 'ranef.R' 'boot.R' 'occuFP.R' 'gpcount.R'
        'occuPEN.R' 'pcount.spHDS.R' 'occuMS.R' 'occuTTD.R'
        'distsampOpen.R' 'multmixOpen.R' 'unmarkedCrossVal.R' 'piFun.R'
        'vif.R' 'makePiFun.R' 'posteriorSamples.R'
LinkingTo: Rcpp, RcppArmadillo
SystemRequirements: GNU make
URL: http://groups.google.com/group/unmarked,
        https://sites.google.com/site/unmarkedinfo/home,
        http://github.com/ianfiske/unmarked,
        http://github.com/rbchan/unmarked
BugReports: https://github.com/rbchan/unmarked/issues
NeedsCompilation: yes
Packaged: 2020-05-03 17:05:22 UTC; ken
Author: Richard Chandler [aut],
  Ken Kellner [aut],
  Ian Fiske [aut],
  David Miller [aut],
  Andy Royle [cre, aut],
  Jeff Hostetler [aut],
  Rebecca Hutchinson [aut],
  Adam Smith [aut],
  Marc Kery [ctb],
  Mike Meredith [ctb],
  Auriel Fournier [ctb],
  Ariel Muldoon [ctb],
  Chris Baker [ctb]
Maintainer: Andy Royle <aroyle@usgs.gov>
Repository: CRAN
Date/Publication: 2020-05-04 11:40:02 UTC
Built: R 3.6.3; x86_64-w64-mingw32; 2020-05-12 08:01:52 UTC; windows
Archs: x64
